#!/usr/bin/bash

"""
This is "PCAPLog", by Eli Fulkerson

This script is used in conjunction with pcaplog.lua, which should be placed
in the appropriate wireshark directory.
(For instance ~/.local/lib/wireshark/plugins)


PCAPLog is run as a 'prefix' to another command, in the same way that the unix "time" command
can be used.   When used in this way, it will send the command line arguments as well as the
individual lines of stdout generated by the command over the network to be collected by Wireshark.

The purpose of this is to allow exact correlation of what-you-typed-in-the-terminal and the
network behavior that was observed in response.

The pcaplog will include the protocol version, mode, the UTC time, a unique hash string for
each command (to aid in correlation, it's annoying to search for date strings), the current
user@hostname and the data itself.


Potential upgrades:
- parse the command line, capture and log hashes/versions of binaries run or payloads sent?
- It would be nice to change the color, but since these packets get an ICMP unreachable
  response, Wireshark will always color them red with the other network errors.
- Some level of encryption - encrypt in python, decrypt in lua.  Not sure how much
  you can do in terms of including additional libraries in the context of the wireshark
  plugin environment.
- Better support for compound linux command lines, pipes, redirects etc.
- More better lua code.

"""


"User editable options"

# The target IP address needs to be directional - that is, traffic sent from your computer needs
# to be routed in the appropriate direction in order to be picked up by your packet capture host.
# (which might not be on the same PC).  Adjust this value if necessary to the correct "over there".
# You can also adjust to an arbitrary number for easier filtering in Wireshark if you desire as
# long as the routing passes over your capture interface (for instance, 127.0.0.1 will never work)

TARGETIP = "8.8.8.8"

# This is the TTL used to send the logging packets                      
# Keep the TTL as low as possible so that you can never reach TARGETIP.  For localhost, this is always 1.
# you can raise this if necessary in order to get the traffic to reach your packet capture host.

TTL = 1

# Do we dispaly the "hash" on stdout on the local machine?  This exists so that you can correlate
# the hash shown on the terminal output with the hash found in the pcap, but it might be spammy/
MODE_DISPLAY_HASH = True

# Do we send the command you typed to the log?
MODE_LOG_CMD = True

# Do we send the stdout returned by that command to the log?
MODE_LOG_STDOUT = True


"Don't edit these"
# Don't edit these options without also modifying the lua plugin to match
PROTOCOLNAME = "PCAPLOG"
VERSION = "1"
DELIM = "\n"          # newline shouldn't appear in a single cmdline...
UDPDISCARD = 9        # "discard" protocol rfc863

import sys
import datetime
import os
import socket
import subprocess
import hashlib
import signal

# handle control-c
def handler(signum, frame):
    sys.exit(1)
signal.signal(signal.SIGINT, handler)


def main():
    # gathering the data we want to send with each logging line
    date = datetime.datetime.now().isoformat()
    ver = f"{PROTOCOLNAME}.{VERSION}"
    user = f"{os.getlogin()}@{socket.gethostname()}"
    cmd = " ".join(sys.argv[1:])
    mode = "cmd"


    #not quite an arg parse - since this is a "prefix" command I want to keep this to an absolute minimum
    # we will still go ahead and run the command, just mark it differently in the pcaplog
    if cmd.split(' ')[0] == 'rem':
       mode = "rem"
       cmd = ' '.join(cmd.split(' ')[1:])

    datehash = "n/a"
    if MODE_DISPLAY_HASH == True:
        # easier to search for, more distinct bits than the date itself
        datehash = hashlib.sha256(date.encode()).hexdigest()
        sys.stdout.write(f"PCAPLog hash : {datehash}\n\n")

    # This is our "cmd" message
    message = bytes(f"{ver}{DELIM}{mode}{DELIM}{date}{DELIM}{datehash}{DELIM}{user}{DELIM}{cmd}", "utf-8")

    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.setsockopt(socket.IPPROTO_IP, socket.IP_TTL, TTL)

    # we don't execute remarks, and always want to send them regardless of MODE_LOG_CMD
    if mode == "rem":
       s.sendto(message, (TARGETIP, UDPDISCARD))
       sys.exit(0)

    if MODE_LOG_CMD == True:
        # fire ze missiles
        s.sendto(message, (TARGETIP, UDPDISCARD))

    # Now we run the command, optionally sending each line of its stdout to the pcaplog as well
    my_env = os.environ.copy()
    mode="out"   # this is just for display purposes in wireshark
    proc = subprocess.Popen(cmd, executable="/bin/bash", shell=True, stdout=subprocess.PIPE, env=my_env)
    for line in proc.stdout:
        line = line.decode('utf-8')
        sys.stdout.write(line)

        if MODE_LOG_STDOUT == True:
            date = datetime.datetime.now().isoformat()  # time has passed, so grab the new timestamp
            datehash = "n/a"  # we aren't including the datehash in the stdout output, it would be too spammy on the console side
            message = bytes(f"{ver}{DELIM}{mode}{DELIM}{date}{DELIM}{datehash}{DELIM}{user}{DELIM}{line}", "utf-8")        
            s.sendto(message, (TARGETIP, UDPDISCARD))
            
    return 0
        
if __name__ == '__main__':
    status = main()
    sys.exit(status)
